from __future__ import annotations

from cleo.io.io import IO
from poetry.plugins.application_plugin import ApplicationPlugin
from poetry.plugins.plugin import Plugin
from poetry.poetry import Poetry

from poetry_git_version_plugin import config
from poetry_git_version_plugin.commands import GitVersionCommand
from poetry_git_version_plugin.exceptions import PluginException, plugin_exception_wrapper
from poetry_git_version_plugin.services import GitVersionService, get_git_tag


class PoetryGitVersionPlugin(Plugin):
    """Плагин определения версии по гит тегу"""

    @plugin_exception_wrapper
    def activate(self, poetry: Poetry, io: IO):
        io.write_line(f'<b>{config.PLUGIN_NAME}</b>: Init')

        self.plugin_config = config.PluginConfig(poetry.pyproject)

        try:
            tag = GitVersionService(io, self.plugin_config).get_tag()

        except Exception as ex:
            if self.plugin_config.ignore_exception:
                io.write_error(f'<b>{config.PLUGIN_NAME}</b>: {ex.args[0]}')
                io.write_error(f'<b>{config.PLUGIN_NAME}</b>: Ignore Exception')
                return

            raise ex

        if tag is not None:
            poetry.package.version = tag

        io.write_line(f'<b>{config.PLUGIN_NAME}</b>: Finished \n')


class PoetryGitVersionApplicationPlugin(ApplicationPlugin):
    commands = [GitVersionCommand]
